# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import ruamel.yaml


def none_representer(dumper, data):
    return dumper.represent_scalar('tag:yaml.org,2002:null', 'null')


class YAML(object):
    def __init__(self):
        self.yaml = ruamel.yaml.YAML(typ='rt')
        self.yaml.width = 256
        self.yaml.allow_duplicate_keys = True
        self.yaml.representer.add_representer(type(None), none_representer)
        self.yaml.indent(mapping=2, sequence=4, offset=2)
        self.yaml.Constructor.add_constructor(
            '!encrypted/pkcs1-oaep',
            ruamel.yaml.SafeConstructor.construct_yaml_seq,
        )

    def load(self, stream):
        return self.yaml.load(stream)

    def dump(self, data, *args, **kwargs):
        self.yaml.dump(data, *args, **kwargs)
